@@ -25,6 +25,7 @@
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_tmp_sup).
+
 -author('alexey@process-one.net').
 
 -export([start_link/2, init/1]).
@@ -32,8 +33,8 @@
 start_link(Name, Module) ->
     supervisor:start_link({local, Name}, ?MODULE, Module).
 
-
 init(Module) ->
-    {ok, {{simple_one_for_one, 10, 1},
-	  [{undefined, {Module, start_link, []},
-	    temporary, brutal_kill, worker, [Module]}]}}.
+    {ok,
+     {{simple_one_for_one, 10, 1},
+      [{undefined, {Module, start_link, []}, temporary,
+	brutal_kill, worker, [Module]}]}}.
